%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the method for computing the Imbalance Ratio formulation presented in the paper Over-sampling the minority class in the feature space published in the IEEE Transactions on Neural Networks and Learning Systems.
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/efso
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [newTrain newTest] = calculateEmpiricalFeatureSpace(kernelMatrixTrain,kernelMatrixTest,percent_dim)
% Compute the empirical feature space
% Input variables: 
% *kernelMatrixTrain: kernel matrix for training.
% *kernelMatrixTest: kernel matrix for test.
% *percent_dim: If n is a number it represents the k value for the k-nn algorithm in the SMOTE method. If n is a matrix it represents the set of patterns to over-sample (two pair of patterns per new synthetic example)
% Output variables
% *newTrain: Train patterns in the empirical feature space.
% *newTest: Test patterns in the empirical feature space.
        
	toleranceEigen = max(size(kernelMatrixTrain))*eps(norm(kernelMatrixTrain));

	% Decompose the kernel matrix
        [V,D] = eig(kernelMatrixTrain);
        a = sum(D);
        indexes = a>toleranceEigen;
	
	numeigenval = numel(indexes);
	% Compute selected dimensions according to percent_dim
	selected = percent_dim*numeigenval;
        indexes(1:(size(V,2)-selected)) = false;
        positiveEigenval = diag(1./sqrt(a(indexes)));
        eigenvectors = V(:,indexes);
      
        product = positiveEigenval*eigenvectors';
	% Compute training and test patterns in the EFS
        newTrain = kernelMatrixTrain'*product';
        newTest = kernelMatrixTest'*product';

end
