%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the method for computing the Imbalance Ratio formulation presented in the paper Over-sampling the minority class in the feature space published in the IEEE Transactions on Neural Networks and Learning Systems.
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/efso
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath tools/
addpath kernelLearning/
addpath tools/libsvm-weights-3.12/matlab/

% Example code for a toy ordinal dataset.
load train
load test

% Boolean variable that indicates whether to conduct the kernel optimisation
optimise_kernel = 1;
% Boolean variable for using a mahalanobis kernel (only considered with opimisation)
mahalanobis_kernel = 1;
% Boolean variable to perform preferential over-sampling
preferential_ov = 0;
% Number of neighbors to consider for the knn method
parameters.n = 5;
% SVC cost parameter for the cost-sensitive algorithm used in preferential over-sampling
parameters.C = 1;
% Kernel parameter for the spherical Gaussian kenel
parameters.k = 0.001;
% Beta values for preferential over-sampling. If b is a vector it represents the values to use for the convex combination. If b is a scalar it refers to the only beta value used for the over-sampling.
parameters.b = [-2 0 1];
% Percentage of dimensions to be used for the EFS construction.
parameters.pDimensions = 0.2;

if optimise_kernel == 0,
	kernelMatrixTrain = computeKernelMatrix(train.patterns', train.patterns',parameters.k);
	kernelMatrixTest = computeKernelMatrix(train.patterns', test.patterns',parameters.k);
else
	if mahalanobis_kernel == 0,
		[solution] = alignSphericalKernel(train);
		kernelMatrixTrain = computeKernelMatrix(train.patterns', train.patterns',solution);
		kernelMatrixTest = computeKernelMatrix(train.patterns', test.patterns',solution);
	else
		[solutions] = alignMahalanobisDistanceKernel(train);

    	sigmas = solutions(1:end-1);
    	alpha = 10.^solutions(end);
    	U = reshape(sigmas,size(train.patterns,2), size(train.patterns,2));
    	Q = U'*U;

		kernelMatrixTrain = computeMahalanobisDistanceKernelMatrixAlpha(train.patterns', train.patterns',Q, alpha);
		kernelMatrixTest = computeMahalanobisDistanceKernelMatrixAlpha(train.patterns', test.patterns',Q, alpha);
	end
end

if preferential_ov == 0,
	[newKernelMatrixTrain, newTrainTargets, trainEFS, newkernelMatrixTest, testEFS] = ksmote(kernelMatrixTrain, train.targets, kernelMatrixTest, parameters);
else
	[newKernelMatrixTrain, newTrainTargets, trainEFS, newKernelMatrixTest, testEFS] = preferentialOversampling(kernelMatrixTrain, train, kernelMatrixTest, parameters);
end

