%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the method for computing the Imbalance Ratio formulation presented in the paper Over-sampling the minority class in the feature space published in the IEEE Transactions on Neural Networks and Learning Systems.
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/efso
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [solutions] = alignMahalanobisDistanceKernel(train)
% Optimise the kernel function with the mahalanobis distance using kernel-target alignment
% Input variables: 
% *train: train structure containing the train patterns and the associated
%  targets
% Output variables
% *solutions: set of parameters chosen by the method designed.
           
	% Compute the ideal train kernel matrix according to the data labels
	ideal_grammatrix = zeros(numel(train.targets),numel(train.targets));
        for i=1:size(train.patterns,1),
        	ideal_grammatrix(i,:) = (train.targets == train.targets(i)); 
        end
        
        seed = 1;
        s = RandStream.create('mt19937ar','seed', seed);
            
        RandStream.setDefaultStream(s);
        
	% Initialise parameters
	aux = inv(cov(train.patterns));        
	initials = reshape(aux,numel(aux),1);
	initials = initials';
        initials = [initials, 0];

        f = @(x,train,ideal_grammatrix)alignmentMahalanobisDistanceDerivativeAlphaNoTriangular(x,train,ideal_grammatrix);
        parm.method = 'IRprop+';
        parm.full_hist = true;
        parm.d_Obj = -1;

        % Optimise the kernel
        [x,e,exitflag,stats] = rprop(f,initials,parm,train.patterns',centerkernel(ideal_grammatrix));
        % Search for the parameter combination with the lowest error
        [m,index] = min(stats.error);
        % Store solutions
        solutions = stats.x{index};
        solutions(end) = 10.^solutions(end);
            
end
