%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the method for computing the Imbalance Ratio formulation presented in the paper Over-sampling the minority class in the feature space published in the IEEE Transactions on Neural Networks and Learning Systems.
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/efso
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [kernelMatrix] = computeMahalanobisDistanceKernelMatrixAlpha(patterns1, patterns2, Q, alpha)
% Compute the kernel matrix for a Gaussian kernel with Mahalanobis distance
% Input variables: 
% *patterns1: training set of patterns
% *patterns2: test set of patterns
% *Q: matrix of hyperparameters
% *alpha: alpha parameter
% Output variables
% *kernelMatrix: Resultant kernel matrix

    Nf1             = size(patterns1, 2);
    Nf2             = size(patterns2, 2);
    kernelMatrix   = ones(Nf1, Nf2);

    for i = 1:Nf2,
        kernelMatrix(:,i)  = exp( -sum( ((patterns1 - repmat(patterns2(:,i),1,Nf1))' *Q) .* (patterns1 - repmat(patterns1(:,i),1,Nf1))',2) /2*alpha^2);
    end

end
