%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the method for computing the Imbalance Ratio formulation presented in the paper Over-sampling the minority class in the feature space published in the IEEE Transactions on Neural Networks and Learning Systems.
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/efso
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [newKernelMatrixTrain, newTrainTargets, trainEFS, newkernelMatrixTest, testEFS] = ksmote(kernelMatrixTrain, trainTargets, kernelMatrixTest, param)
% Perform over-sampling in the empirical feature space
% Input variables: 
% *kernelMatrixTrain: kernel matrix for training.
% *trainTargets: training targets
% *kernelMatrixTest: kernel matrix for test.
% *param: struct of parameters for the algorithm. param.pDimensions corresponds to the percentage of dimensions to be selected for the EFS. param.n refers to the k in the knn method used for the over-sampling.
% Output variables
% *newKernelMatrixTrain: Reconstructed train kernel matrix, including information from synthetic data.
% *newTrainTargets: Vector of final targets, incorporating the label of new synthetic data.
% *trainEFS: Train patterns in the empirical feature space.
% *newkernelMatrixTest: Reconstructed test kernel matrix, including information from synthetic data.
% *testEFS: Test patterns in the empirical feature space.

	% Compute empirical feature space and apply SMOTE
        [featureSpaceTrain featureSpaceTest] = calculateEmpiricalFeatureSpace(kernelMatrixTrain,kernelMatrixTest,param.pDimensions);
        [newKernelMatrixTrain,newTrainTargets,sample] = applySMOTE(featureSpaceTrain,trainTargets,param.n); 

	% Reconstruct the kernel matrix
	k1 = featureSpaceTest * sample';
        a = featureSpaceTrain * featureSpaceTest';
        newkernelMatrixTest = [a; k1'];     
	trainEFS = [featureSpaceTrain; sample];
	testEFS = featureSpaceTest;
      
end
