%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the method for computing the Imbalance Ratio formulation presented in the paper Over-sampling the minority class in the feature space published in the IEEE Transactions on Neural Networks and Learning Systems.
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/efso
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sample=probSMOTE(T,N,pairs)
% Implement SMOTE algorithm with fixed patterns to over-sample.
% Input variables: 
% *T: Minority class training patterns.
% *N: Number of patterns to over-sample.
% *pairs: pairs of patterns to be used to create the new synthetic ones.
% Output variables
% *sample: New synthetic data.

weights = rand(1,N);
for i=1:N,
    sample(:,i) = T(:,pairs(1,i)) + weights(i)*(T(:,pairs(2,i)) - T(:,pairs(1,i)));
end


end

